#   ticket.pl - Bibliothek fuer das Infodrom Trouble Ticket System
#   Copyright (c) 1997,8  Martin Schulze <joey@infodrom.north.de>

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

######################################################################
######################################################################

# noc_connect
#   Stellt die Verbindung zur Datenbank her
#
# ticket_init
#   Initialisiert das Ticket-System
#
# stringdate
#   Gibt den heutigen Tag als Zeichenkette zurueck.
#
# shortdate
#   Gibt den heutigen Tag als Zeichenkette zurueck.
#
# gettext
#   Returns a string, possibly in a different language
#
# error_query
#   Fehlermeldung, der fehlgeschlagene Query wird angezeigt.
#
# string_to_date
#   Wandelt einen Datumsstring wie er in der Datenbank gespeichert wird
#   in eine lesbare Form um.
#
# date_to_string
#   Wandelt einen lesbaren Datumsstring in die Form um, in der er in der
#   Datenbank gespeichert werden kann.
#
# index_user_hashes
#   Fuellt diverse Cache-Listen mit Userdaten
#   %ticket_list_user_realname    Zuordnung User Realname
#   %ticket_list_user_email       Zuordnung User Email
#   %ticket_list_email_realname   Zuordnung Email Realname
#   %ticket_list_email_user       Zuordnung Email User
#   %ticket_list_user_groups      Zuordnung User Gruppen
#   %ticket_list_user_admin       Zuordnung User Admin
#   %ticket_list_user_primary     Zuordnung User Primary Group
#
# index_user
#   Fuellt diverse Cache-Listen mit Userdaten.  Stuetzt sich auf
#   index_user_hashes ab.
#
# get_language
#   Gibt die zu benutzende Sprache zurueck, beinhatet ein index_user()
#
# user_email
#   Gibt die Email-Adresses eines Users zurueck, falls unbekannt, wird
#   ein leerer String zurueckgegeben.
#
# user_groups
#   Gibt die Gruppen eines Users zurueck, falls unbekannt, wird ein
#   leerer String zurueckgegeben.
#
# user_realname
#   Gibt den Realname einers Users zurueck, falls unbekannt, wird der
#   Username zurueckgegeben.
#
# user_primary
#   Gibt die primaere Gruppe einers Users zurueck, falls unbekannt, wird der
#   Username zurueckgegeben.
#
# email_realname
#   Gibt den Realname eines Users mit der angegebenen Email-Adresse
#   zurueck, falls unbekannt, wird die Email-Adresse zurueckgegeben.
#
# email_user
#   Gibt den Benutzer zur angegebenen Email-Adresse zurueck, falls unbekannt,
#   wird die Email-Adresse zurueckgegeben.
#
# is_admin
#   Gibt 0 oder 1 zurueck, je nachdem, ob der angegebene User als Admin
#   eingetragen ist oder nicht.
#
# is_user
#   Gibt 0 oder 1 zurueck, je nachdem, ob der angegebene User
#   tatsaechlich als User (in der angegebenen Gruppe) existiert.
#
# user_assoc
#   Gibt die assoziierten Gruppen eines Users zurueck, falls unbekannt, wird ein
#   leerer String zurueckgegeben.
#
# uniq_fields
#   Loescht doppelte Eintraege aus der Liste - zur Vermeidung von
#   doppelten Mails.
#
# uniq_user_email
#    Returns a uniqe list of email addresses regarding the given array of
#    users.
#
# uniq_user_realname
#    Returns a uniqe list of realnames regarding the given array of
#    users.
#
# groupname
#    Gibt die Beschreibung zu einer Gruppe zurueck, ggf. wird ein Query
#    abgesetzt
#
# get_subject
#   Gibt das Subject eines Tickets zurueck.
#
# prepare_text
#   Tries to do some conversation from normal text to html'ed
#
# de_html
#   Tries to remove some tags from texts.
#
# de_msql
#   Tries to remove some msql quotes.
#
# load_language
#   Loads strings for the given language.
#
# start_program
#   Gibt 1 zurueck, wenn der angegebene User das angegebene Programm
#   ausfuehren darf, ansonsten 0.
#
# forkme
#   Returns 0 if it's the child, $pid if its the parent and undef if error.
#
# sql_clike
#   Converts a string into an expression to be used by the clike expression.
#
# html_quote
#   Converts a string into proper HTML text.  This is essentially needed
#   if text fields contain regular quotes (") which are used to delimit
#   fields as well.

push (@INC, "/etc/noc") if (!grep (/\/etc\/noc/, @INC));
push (@INC, "/usr/lib/ticket") if (!grep (/\/usr\/lib\/ticket/, @INC));
require 'ticket-db.pl';
require 'ticket-config.pl';

my $dbh;

########### noc_connect
#
# Stellt die Verbindung zur Datenbank her
#
sub noc_connect
{
    my $tick_engine;

    if ($ticket_dbdriver eq 'Pg') {
	$tick_engine  = "dbi:$ticket_dbdriver:dbname=$ticket_dbase";
    } elsif ($ticket_dbdriver eq 'mSQL') {
	$tick_engine  = "dbi:$ticket_dbdriver:$ticket_dbase";
	$tick_engine .= "@$ticket_dbhost" if ($ticket_dbhost);
    } elsif ($ticket_dbdriver eq 'mysql') {
	$tick_engine  = "dbi:$ticket_dbdriver:$ticket_dbase";
    }

    if (defined $ticket_username) {
	if (defined $ticket_password) {
	    $dbh = DBI->connect($tick_engine, $ticket_username, $ticket_password);
	} else {
	    $dbh = DBI->connect($tick_engine, $ticket_username);
	}
    } else {
	$dbh = DBI->connect($tick_engine);
    }
    if (!$dbh) {
	print "<h1>Access to database denied!</h1>\n\n";
	return 0;
    }
    return $dbh
}

########### ticket_init
#
# Initialisiert das Ticket-System
#
# [0] REMOTE_USER
#
sub ticket_init
{
    my $sth;

    $ENV{'REMOTE_USER'} = $ENV{'LOGNAME'} if (!scalar $ENV{'REMOTE_USER'});

    $dbh = &noc_connect if (!$dbh);

    if ($ticket_dbdriver eq 'Pg') {
	$ticket_clike = "~*";
    } elsif ($ticket_dbdriver eq 'mysql') {
	$ticket_clike = "LIKE";
    } elsif ($ticket_dbdriver eq 'mSQL') {
	$ticket_clike = "CLIKE";
    }

#    $sth = $dbh->query("SELECT subject,body,closing FROM $ticket_table WHERE nr = 0");
#
#    ($ticket_length{'subject'},$ticket_length{'body'},$ticket_length{'closing'}) = $sth->length;      
#
#   Moved into ticket-config.pl temporarily since I don't know how to
#   determine the size of field with DBI

    &load_language(&get_language($_[0]));

    if (!&start_program("ticket", $_[0])) {
	print "<h1>" . &gettext("Access denied.") . "</h1>\n\n";
	return 0;
    }

    if ($ENV{'HTTP_USER_AGENT'} =~ /Lynx/i) {
	$ticket_browser = 'Lynx';
	$ticket_browser_table = undef;
    } elsif ($ENV{'HTTP_USER_AGENT'} =~ /Mosaic/i) {
	$ticket_browser = 'Mosaic';
	$ticket_browser_table = undef;
    } elsif ($ENV{'HTTP_USER_AGENT'} =~ /Grail/i) {
	$ticket_browser = 'Grail';
	$ticket_browser_table = undef;
    } elsif ($ENV{'HTTP_USER_AGENT'} =~ /Chimera/i) {
	$ticket_browser = 'Chimera';
	$ticket_browser_table = undef;
    } elsif ($ENV{'HTTP_USER_AGENT'} =~ /Mozilla|MSIE/i) {
	$ticket_browser = 'Mozilla';
	$ticket_browser_table = 1;
    }

    return $dbh;
}

########### stringdate
#
# Gibt den heutigen Tag als Zeichenkette zurueck.
#
sub stringdate
{
    ($date_sec,$date_min,$date_hour,$date_mday,$date_mon,$date_year,$date_wday,$date_isdst)
	= localtime();
    return sprintf ("%4d%02d%02d", $date_year+1900, $date_mon+1, $date_mday);
}

########### shortdate
#
# Gibt den heutigen Tag als Zeichenkette zurueck.
#
sub shortdate
{
    ($date_sec,$date_min,$date_hour,$date_mday,$date_mon,$date_year,$date_wday,$date_isdst)
	= localtime();
    return sprintf ("%d.%d.%d", $date_mday, $date_mon+1, $date_year);
}

########### gettext
#
# Returns a string, possibly in a different language
sub gettext
{
    my $text = $language{$_[0]};
    return $text if ($text);
    return $_[0];
}

########### error_query
#
# Fehlermeldung, der fehlgeschlagene Query wird angezeigt.
#
sub error_query
{
    my $query = $_[0];

    printf "<h3>%s</h3>\n", &gettext("An SQL-Query failed!");
    printf "<h3>%s</h3>\n\n", &gettext("Please press 'Reload' or contact the maintainer.");
    printf "<pre>\n%s\n</pre>\n\n", $query;
    printf "<blockquote>\n%s\n</blockquote>\n\n", $query;
}

########### string_to_date
#
# Wandelt einen Datumsstring wie er in der Datenbank gespeichert wird
# in eine lesbare Form um.
#
sub string_to_date
{
    return "" if (!$_[0]);

    $_[0] =~ /(....)(..)(..)/;
    return "$3.$2.$1";
}

########### date_to_string
#
# Wandelt einen lesbaren Datumsstring in die Form um, in der er in der
# Datenbank gespeichert werden kann.
#
sub date_to_string
{
    return "" if (!$_[0]);

    ($date_sec,$date_min,$date_hour,$date_mday,$date_mon,$date_year,$date_wday,$date_isdst)
	= localtime();

    if ($_[0] eq "heute" || $_[0] eq "sofort" || $_[0] eq "pronto" || $_[0] eq "today" || $_[0] eq "now") {
	$day = $date_mday;
	$mon = $date_mon+1;
	$year = $date_year;
    } elsif ($_[0] eq "gestern" || $_[0] eq "yesterday") {
	$day = $date_mday-1 if ($date_mday);
	$mon = $date_mon+1;
	$year = $date_year;
    } elsif ($_[0] eq "morgen" || $_[0] eq "tomorrow") {
	$day = $date_mday+1;
	$mon = $date_mon+1;
	$year = $date_year;
    } else {
	($day,$mon,$year) = split(/\./, $_[0]);
	if (!$year) {    
	    $year = $date_year;
	}
    }
    $year += 1900 if ($year < 1901);
    return sprintf("%4d%02d%02d", $year,$mon,$day);
}

########### index_user_hashes
#
# Fuellt diverse Cache-Listen mit Userdaten
#
# %ticket_list_user_realname	Zuordnung User Realname 
# %ticket_list_user_email	Zuordnung User Email
# %ticket_list_email_realname	Zuordnung Email Realname
# %ticket_list_email_user	Zuordnung Email User
# %ticket_list_user_groups	Zuordnung User Gruppen
# %ticket_list_user_admin	Zuordnung User Admin
# %ticket_list_user_primary	Zuordnung User Primary Group
#
#
# Args = (realname,email,groups,admin)
sub index_user_hashes
{
    my @row = @_;

    $ticket_list_user_realname{$row[0]} = $row[1];
    $ticket_list_user_email{$row[0]} = $row[2];
    $ticket_list_email_realname{$row[2]} = $row[1];
    $ticket_list_email_user{$row[2]} = $row[0];
    $ticket_list_user_groups{$row[0]} = $row[3];
    if ($row[4] eq "j") {
	$ticket_list_user_admin{$row[0]} = 1;
    } else {
	$ticket_list_user_admin{$row[0]} = 0;
    }
    $ticket_list_user_primary{$row[0]} = $row[5];
}

########### index_user
#
# Fuellt diverse Cache-Listen mit Userdaten.  Stuetzt sich auf
# index_user_hashes ab.
#
sub index_user
{
    my $user = shift;
    my $query;
    my $sth;
    my $rc;
    my @row;

    $dbh = &noc_connect if (!$dbh);

    $query  = "SELECT username,realname,email,groups,admin,maingroup FROM $ticket_table_user";
    $query .= sprintf (" WHERE username = '%s' OR email = '%s'", $user, $user) if (length($user));

    $sth = $dbh->prepare($query);
    if ($sth) {
	$rc = $sth->execute;
	if ($rc > 0 && (@row = $sth->fetchrow_array)) {
	    &index_user_hashes($row[0],$row[1],$row[2],$row[3],$row[4],$row[5]);
	} else {
	    printf STDERR "Query: %s\n", $query,;
	}
    }
}

########### get_language
#
# Gibt die zu benutzende Sprache zurueck, beinhaltet ein index_user()
#
sub get_language
{
    my $user = shift;
    my $lang = $ticket_default_language;
    my $query;
    my $sth;
    my $rc;
    my @row;

    $dbh = &noc_connect if (!$dbh);

    $query  = "SELECT username,realname,email,groups,admin,maingroup,language,programs,assoc,misc FROM $ticket_table_user";
    $query .= sprintf (" WHERE username = '%s'", $user) if (length($user));

    $sth = $dbh->prepare($query);
    if ($sth) {
	$rc = $sth->execute;
	if ($rc > 0 && (@row = $sth->fetchrow_array)) {
	    &index_user_hashes($row[0],$row[1],$row[2],$row[3],$row[4],$row[5]);
	    $lang = $row[6];
	    $ticket_intern{'user'} = $user;
	    $ticket_intern{'programs'} = $row[7];
	    $ticket_intern{'assoc'} = $row[8];
	    $ticket_intern{'misc'} = $row[9];
	}
    }
    return $lang;
}

########### user_email
#
# Gibt die Email-Adresses eines Users zurueck, falls unbekannt, wird
# ein leerer String zurueckgegeben.
#
sub user_email
{
    return "" if (!$_[0]);
    return $ticket_list_user_email{$_[0]} if (exists($ticket_list_user_email{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_user_email{$_[0]} if (exists($ticket_list_user_email{$_[0]}));
    return "";
}

########### user_groups
#
# Gibt die Gruppen eines Users zurueck, falls unbekannt, wird ein
# leerer String zurueckgegeben.
#
sub user_groups
{
    return "" if (!$_[0]);
    return $ticket_list_user_groups{$_[0]} if (exists($ticket_list_user_groups{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_user_groups{$_[0]} if (exists($ticket_list_user_groups{$_[0]}));
    return "";
}

########### user_realname
#
# Gibt den Realname einers Users zurueck, falls unbekannt, wird der
# Username zurueckgegeben.
#
sub user_realname
{
    return "" if (!$_[0]);
    return $ticket_list_user_realname{$_[0]} if (exists($ticket_list_user_realname{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_user_realname{$_[0]} if (exists($ticket_list_user_realname{$_[0]}));
    return $_[0];
}

########### user_primary
#
# Gibt die primaere Gruppe einers Users zurueck, falls unbekannt, wird 
# ein leerer String zurueckgegeben.
#
sub user_primary
{
    return "" if (!$_[0]);
    return $ticket_list_user_primary{$_[0]} if (exists($ticket_list_user_primary{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_user_primary{$_[0]} if (exists($ticket_list_user_primary{$_[0]}));
    return "";
}

########### email_realname
#
# Gibt den Realname eines Users mit der angegebenen Email-Adresse
# zurueck, falls unbekannt, wird die Email-Adresse zurueckgegeben.
#
sub email_realname
{
    return if (!$_[0]);
    return $ticket_list_email_realname{$_[0]} if (exists($ticket_list_email_realname{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_email_realname{$_[0]} if (exists($ticket_list_email_realname{$_[0]}));
    return $_[0];
}

########### email_user
#
# Gibt den User zur angegebenen Email-Adresse zurueck, falls unbekannt,
# wird die Email-Adresse zurueckgegeben.
#
sub email_user
{
    return if (!$_[0]);
    return $ticket_list_email_user{$_[0]} if (exists($ticket_list_email_user{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_email_user{$_[0]} if (exists($ticket_list_email_user{$_[0]}));
    return $_[0];
}

########### is_admin
#
# Gibt 0 oder 1 zurueck, je nachdem, ob der angegebene User als Admin
# eingetragen ist oder nicht.
#
sub is_admin
{
    return 0 if (!$_[0]);
    return $ticket_list_user_admin{$_[0]} if (exists($ticket_list_user_admin{$_[0]}));
    &index_user ($_[0]);
    return $ticket_list_user_admin{$_[0]} if (exists($ticket_list_user_admin{$_[0]}));
    return 0;
}

########### is_user
#
# Gibt 0 oder 1 zurueck, je nachdem, ob der angegebene User
# tatsaechlich als User (in der angegebenen Gruppe) existiert.
#
sub is_user
{
    my $group = $_[1];

    return 0 if (!$_[0]);
    return 1 if (exists($ticket_list_user_groups{$_[0]}) && $ticket_list_user_groups{$_[0]} =~ /$group/);
    &index_user ($_[0]);
    return 1 if (exists($ticket_list_user_groups{$_[0]}) && $ticket_list_user_groups{$_[0]} =~ /$group/);
    return 0;
}

########### user_assoc
#
# Gibt die assoziierten Gruppen eines Users zurueck, falls unbekannt, wird ein
# leerer String zurueckgegeben.
#
sub user_assoc
{
    my $user = shift;
    my $query;
    my $sth;
    my $rc;
    my @row;

    return "" if (!$user);

    if (exists($ticket_intern{'user'}) && exists($ticket_intern{'assoc'})
	&& $ticket_intern{'user'} eq $_[0]) {
	return $ticket_intern{'assoc'};
    } else {
	$dbh = &noc_connect if (!$dbh);

	$query .= sprintf ("SELECT assoc FROM %s WHERE username = '%s' OR email = '%s'", $ticket_table_user, $user, $user);

	$sth = $dbh->prepare($query);
	if ($sth) {
	    $rc = $sth->execute;
	    if ($rc > 0 && (@row = $sth->fetchrow_array)) {
		return $row[0];
	    }
	}
    }
    return "";
}

########### uniq_fields
#
# Loescht doppelte Eintraege aus der Liste - zur Vermeidung von
# doppelten Mails.
#
sub uniq_fields
{
    my %addr = ();
    my $user;
    my @result = ();

    while ($user = shift) {
	if ($user && length($user)) {
	    if (!exists $addr{$user}) {
		$addr{$user} = 1;
		push (@result, $user);
	    }
	}
    }
    return @result
}

########### uniq_user_email
#
# Returns a uniqe list of email addresses regarding the given array of
# users.
#
sub uniq_user_email
{
    my @user = uniq_fields(@_);
    my @fields = ();
    my $user;
    my $addr;

    while ($user = shift (@user)) {
	$addr = &user_email($user);
	push(@fields, $addr) if ($addr);
    }
    return @fields;
}

########### uniq_user_realname
#
# Returns a uniqe list of realnames regarding the given array of
# users.
#
sub uniq_user_realname
{
    my @user = uniq_fields(@_);
    my @fields = ();
    my $user;
    my $addr;

    while ($user = shift (@user)) {
	$addr = &user_realname($user);
	push(@fields, $addr) if ($addr);
    }
    return @fields;
}

########### groupname
#
#  Gibt die Beschreibung zu einer Gruppe zurueck, ggf. wird ein Query
#  abgesetzt

#
sub groupname
{
    my $query;
    my $sth;
    my $rc;
    my @row;

    return '' if (!$_[0]);
    return $groupname{$_[0]} if (exists($groupname{$_[0]}));
    $dbh = &noc_connect if (!$dbh);

    $query  = "SELECT description FROM $ticket_table_groups";
    $query .= sprintf (" WHERE name = '%s'", $_[0]);

    $sth = $dbh->prepare($query);
    if ($sth) {
	$rc = $sth->execute;
	if ($rc > 0 && (@row = $sth->fetchrow_array)) {
	    $groupname{$_[0]} = $row[0];
	    return $row[0];
	}
    }
    return $_[0];
}

########### get_subject
#
# Gibt das Subject eines Tickets zurueck.
#
sub get_subject
{
    my $query;
    my $sth;
    my $rc;
    my @row;

    return if (!$_[0]);

    $dbh = &noc_connect if (!$dbh);

    $query = "SELECT subject FROM $ticket_table WHERE nr = $_[0]";
    $sth = $dbh->prepare($query);
    if ($sth) {
	$rc = $sth->execute;
	if ($rc > 0 && (@row = $sth->fetchrow_array)) {
	    return $row[0];
	} else {
	    do error_query($query);
	}
    } else {
	do error_query($query);
    }
}

########### prepare_text
#
# Tries to do some conversation from normal text to html'ed
#
sub prepare_text
{
    my $text = $_[0];

    $text =~ s/\r//g;
    $text =~ s/\n\n(\n)*/<p>\n/g;
    return $text;
}

########### de_html
#
# Tries to remove some tags from texts.
#
sub de_html
{
    my $text = $_[0];

    $text =~ s/<p>/\n/g;
    $text =~ s/(\r?\n)*<hr>(\r?\n)*/\n--------------------------------------------------------------------\n/g;
    $text =~ s/<li>/- /g;
    $text =~ s/<..?.?.?>//g;
    $text =~ s/&Auml;/Ae/g;
    $text =~ s/&auml;/ae/g;
    $text =~ s/&Ouml;/Oe/g;
    $text =~ s/&ouml;/oe/g;
    $text =~ s/&Uuml;/Ue/g;
    $text =~ s/&uuml;/ue/g;
    $text =~ s/&szlig;/ss/g;
    return $text;
}

########### de_msql
#
# Tries to remove some msql quotes.
#
sub de_msql
{
    my $text = $_[0];

    $text =~ s/\\'/'/g;
    $text =~ s/\\\\/\\/g;
    $text =~ s/<([^<>]*)\@([^<>]*)>/&lt;$1\@$2&gt;/g;
    $text =~ s:\[([^/\]]*)/([^\]]*)\]:<b>Update from \1 by \2</b><p>:g;
    # The following regexp is stolen from the Debian package of urlview.
    $text =~ s!((((http|https|ftp|gopher)|mailto):(//)?[^ <>"\t\r\n]*|www\.[-a-z0-9.]+)[^ .,;\t\r\n<">\):])!<a href=\"$1\">$1</a>!g;
    return $text;
}

########### load_language
#
# Load strings for the given language
#
sub load_language
{
    my $lang = $_[0];

    require "ticket-$lang.pl" if ($lang ne "gbr");
}

########### start_program
#
# Gibt 1 zurueck, wenn der angegebene User das angegebene Programm
# ausfuehren darf, ansonsten 0.
#
sub start_program
{
    my $query;
    my $sth;
    my $rc;
    my @row;
    my $programs;

    $dbh = &noc_connect if (!$dbh);

    return 1 if ($ENV{'LOGNAME'} eq $ticket_superuser);

    if ( !exists($ticket_intern{'user'}) || !exists($ticket_intern{'programs'})
	|| $ticket_intern{'user'} ne $_[1]) {
	$query = sprintf("SELECT programs FROM $ticket_table_user WHERE username = '%s'", $_[1]);
	$sth = $dbh->prepare($query);
	if ($sth) {
	    $rc = $sth->execute;
	    if ($rc > 0 && (@row = $sth->fetchrow_array)) {
		$programs = $row[0];
	    }
	}
    } else {
	$programs = $ticket_intern{'programs'}
    }

    return 1 if ($programs =~ /:$_[0]:/);

    return 0;
}

########### forkme
#
# Returns 0 if it's the child, $pid if its the parent and undef if
# error.  Possible argument $care, define if the routine needs to be
# successful with fork, ignore if not.
#
sub forkme
{
    my $care = shift;
    my $pid;

    $SIG{'CHLD'} = 'IGNORE';
    if ($pid = fork) {
	waitpid($pid,0);
	return $pid;
    } elsif (defined $pid) {
	if (fork) {
	    exit (0);
	} else {
	    return 0;
	}
    }
    return undef;
}

########### sql_clike
#
# Depending on the backend database the string expression to be used
# by a CLIKE statement varies.  This routine take this into account.
#
sub sql_clike
{
    my $string = shift;

    return "" if (!$string);

    if ($ticket_dbdriver eq 'Pg') {
	return $string;
    } elsif ($ticket_dbdriver eq 'mSQL' || $ticket_dbdriver eq 'mysql') {
	return "%" . $string . "%";
    }
}

########### html_quote
#   Converts a string into proper HTML text.  This is essentially needed
#   if text fields contain regular quotes (") which are used to delimit
#   fields as well.
#
sub html_quote
{
    my $string = shift;

    return "" if (!$string);

    $string =~ s/"/\&quot;/g;
    return $string;
}

# Make perl happy
#
1;

# Local variables:
# mode: perl
# End:
